package com.diagbot.facade;

import com.alibaba.fastjson.JSON;
import com.diagbot.client.AIServiceClient;
import com.diagbot.client.bean.CalculateData;
import com.diagbot.client.bean.GdbResponse;
import com.diagbot.client.bean.Response;
import com.diagbot.client.bean.ResponseData;
import com.diagbot.dto.LisResult;
import com.diagbot.entity.ScaleContent;
import com.diagbot.entity.SysLog;
import com.diagbot.enums.ScaleTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.ScaleContentService;
import com.diagbot.util.DateUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.SearchVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 *  数据处理业务层
 * </p>
 *
 * @author Mark Huang
 * @since 2019-03-04
 */
@Component
public class ClinicalFacade {

    @Autowired
    private AIServiceClient aiServiceClient;
    @Autowired
    private ScaleContentService scaleContentService;

    private static String up = "增高";
    private static String down = "降低";
    private static String normal = "正常";

    private static String pos = "阳性";
    private static String neg = "阴性";

    /**
     * 处理临床数据
     *
     * @param searchVo
     * @return ResponseData
     */
    public ResponseData processClinicalData(@RequestBody SearchVo searchVo) {
        SearchVo sData = searchVo;

        sData.setLisArr(processLis(sData.getLisArr()));

        if (ListUtil.isNotEmpty(sData.getLisArr())) {
            List<String> otherVal = sData.getLisArr().stream().map(lisArr -> lisArr.getOtherValue()).collect(Collectors.toList());
            sData.setLis(String.join(",", otherVal));
        }

        Response<ResponseData> res = aiServiceClient.bayesPageData(sData);
        if (res == null || res.getData() == null) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "AI没有返回结果");
        }

        return res.getData();
    }

    private List<LisResult> processLis(List<LisResult> lisArr) {
        if (ListUtil.isEmpty(lisArr)) {
            return lisArr;
        }

        String Otherval = "";

        for (int i = 0; i < lisArr.size(); i++) {
            LisResult lisres = lisArr.get(i);

            Otherval = (lisres.getOtherValue().trim().length() > 0) ? lisres.getOtherValue().trim() + "\n" : "";
            Otherval = lisres.getOtherValue();

            if (Otherval.indexOf(pos) >= 0 || Otherval.indexOf(neg) >= 0) {
                lisres.setOtherValue(lisres.getDetailName() + Otherval);
            } else {
                Otherval = (Otherval.trim().length() > 0) ? Otherval.trim() + "\n" : "";
                if (lisres.getValue() == null) {
                    continue;
                } else if (lisres.getMaxValue() != null && lisres.getValue() > lisres.getMaxValue()) {
                    lisres.setOtherValue(Otherval + lisres.getDetailName() + up);
                } else if (lisres.getMinValue() != null && lisres.getValue() < lisres.getMinValue()) {
                    lisres.setOtherValue(Otherval + lisres.getDetailName() + down);
                } else {
                    lisres.setOtherValue(Otherval + lisres.getDetailName() + normal);
                }
            }
        }

        return lisArr;
    }

    /**
     * 获取量表内容
     *
     * @param searchVo
     * @return
     */
    public Map<String, Object> scale(SearchVo searchVo) {
        Map<String, Object> response = aiServiceClient.scale(searchVo);
        if (null == response) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "量表没有返回结果");
        }
        return response;
    }


    /**
     * 获取量表内容
     *
     * @param searchVo
     * @return
     */
    public List<ScaleContent> getScale(SearchVo searchVo) {
        Map<String, Object> response = aiServiceClient.scale(searchVo);
        if (null == response) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "量表没有返回结果");
        }
        List<ScaleContent> list = getContent(searchVo.getScaleName(), JSON.toJSONString(response));
        return list;
    }


    public List<ScaleContent> getContent(String scaleName, String pushContent) {
        List<ScaleContent> list = scaleContentService.getContentByName(scaleName);
        for (ScaleContent scaleContent : list) {
            if (ScaleTypeEnum.PUSH.getKey() == scaleContent.getType()) {
                scaleContent.setContent(pushContent);
                break;
            }
        }
        return list;
    }

    /**
     * 高危诊断
     *
     * @param searchVo
     * @return
     */
    public GdbResponse highRiskPageData(SearchVo searchVo) {
        Response<GdbResponse> response = aiServiceClient.highRiskPageData(searchVo);
        if (response == null || response.getData() == null) {
            return null;
            //throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "高危没有返回结果");
        }
        return response.getData();
    }

    /**
     * 计算接口
     *
     * @param calculateData
     * @return
     */
    public Map<String, Object> scaleCalc(CalculateData calculateData) {
        Map<String, Object> response = aiServiceClient.scaleCalc(calculateData);
        if (response == null) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "计算没有返回结构");
        }
        return response;
    }
}